/*
 * Copyright 2009 JBoss, a divison Red Hat, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.errai.container;

import javassist.util.proxy.ProxyFactory;
import org.jboss.weld.bootstrap.api.Bootstrap;
import org.jboss.weld.bootstrap.api.Environments;
import org.jboss.weld.context.api.BeanStore;
import org.jboss.weld.context.api.helpers.ConcurrentHashMapBeanStore;
import org.jboss.weld.environment.servlet.Listener;
import org.jboss.weld.environment.servlet.deployment.ServletDeployment;
import org.jboss.weld.environment.servlet.services.ServletResourceInjectionServices;
import org.jboss.weld.environment.servlet.services.ServletServicesImpl;
import org.jboss.weld.environment.servlet.util.Reflections;
import org.jboss.weld.environment.tomcat.WeldAnnotationProcessor;
import org.jboss.weld.injection.spi.ResourceInjectionServices;
import org.jboss.weld.manager.api.WeldManager;
import org.jboss.weld.servlet.api.ServletListener;
import org.jboss.weld.servlet.api.ServletServices;
import org.jboss.weld.servlet.api.helpers.ForwardingServletListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletContextEvent;

/**
 * @author: Heiko Braun <hbraun@redhat.com>
 * @date: Aug 2, 2010
 */
public class DevModeCDIBootstrap extends ForwardingServletListener
{
  private static final Logger log = LoggerFactory.getLogger(DevModeCDIBootstrap.class);

   private static final String BOOTSTRAP_IMPL_CLASS_NAME = "org.jboss.weld.bootstrap.WeldBootstrap";
   private static final String WELD_LISTENER_CLASS_NAME = "org.jboss.weld.servlet.WeldListener";
   private static final String APPLICATION_BEAN_STORE_ATTRIBUTE_NAME = Listener.class.getName() + ".applicationBeanStore";

   private final transient Bootstrap bootstrap;
   private final transient ServletListener weldListener;
   private WeldManager manager;

   public DevModeCDIBootstrap()
   {
      try
      {
         bootstrap = Reflections.newInstance(BOOTSTRAP_IMPL_CLASS_NAME);
      }
      catch (IllegalArgumentException e)
      {
         throw new IllegalStateException("Error loading Weld bootstrap, check that Weld is on the classpath", e);
      }
      try
      {
         weldListener = Reflections.newInstance(WELD_LISTENER_CLASS_NAME);
      }
      catch (IllegalArgumentException e)
      {
         throw new IllegalStateException("Error loading Weld listener, check that Weld is on the classpath", e);
      }
   }

   @Override
   public void contextDestroyed(ServletContextEvent sce)
   {
      bootstrap.shutdown();
      try
      {
         Reflections.classForName("org.apache.AnnotationProcessor");
         sce.getServletContext().removeAttribute(WeldAnnotationProcessor.class.getName());
      }
      catch (IllegalArgumentException e) {}
      super.contextDestroyed(sce);
   }

   @Override
   public void contextInitialized(ServletContextEvent sce)
   {
      // Make Javassist always use the TCCL to load classes
      ProxyFactory.classLoaderProvider = new ProxyFactory.ClassLoaderProvider()
      {
         public ClassLoader get(ProxyFactory pf)
         {
            return Thread.currentThread().getContextClassLoader();
         }

      };
      BeanStore applicationBeanStore = new ConcurrentHashMapBeanStore();
      sce.getServletContext().setAttribute(APPLICATION_BEAN_STORE_ATTRIBUTE_NAME, applicationBeanStore);

      ServletDeployment deployment = new ServletDeployment(sce.getServletContext());
      try
      {
    	  deployment.getWebAppBeanDeploymentArchive().getServices().add(
    	        ResourceInjectionServices.class, new ServletResourceInjectionServices() {});
      }
      catch (NoClassDefFoundError e)
      {
    	 // Support GAE
    	 log.warn("@Resource injection not available in simple beans");
      }

      deployment.getServices().add(ServletServices.class,
            new ServletServicesImpl(deployment.getWebAppBeanDeploymentArchive()));

      bootstrap.startContainer(Environments.SERVLET, deployment, applicationBeanStore).startInitialization();
      manager = bootstrap.getManager(deployment.getWebAppBeanDeploymentArchive());

      //bootstrap.deployBeans().validateBeans().endInitialization();

     log.info("Bean validation disabled in hosted mode");
     bootstrap.deployBeans().endInitialization();

      super.contextInitialized(sce);
   }

   @Override
   protected ServletListener delegate()
   {
      return weldListener;
   }
}
